/*
 *
 *   Copyright (c) International Business Machines  Corp., 2002
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * 	Module: evmsccm.c
 */


/* 
 * this program ensures that evms commands do not run before 
 * ccm has recovered. It is called in evms_failover script 
 */

#include <ocf/oc_event.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>


oc_ev_t *ev_token;

extern void oc_ev_special(const oc_ev_t *, oc_ev_class_t , int );


static void 
my_ms_events(oc_ed_t event, void *cookie, 
		size_t size, const void *data)
{
	/* exit only when ccm has a new membership 	*/
	/* NOTE: OC_EV_MS_INVALID is a new membership   */
	/* event with no quorum 			*/
	if(event == OC_EV_MS_NEW_MEMBERSHIP ||
			event == OC_EV_MS_INVALID) {
		exit(0);
	}
	oc_ev_callback_done(cookie);
}

int
main(int argc, char *argv[])
{
	int ret;
	fd_set rset;
	int	my_ev_fd;
	int  i;

	oc_ev_register(&ev_token);

	oc_ev_set_callback(ev_token, OC_EV_MEMB_CLASS, my_ms_events, NULL);
	oc_ev_special(ev_token, OC_EV_MEMB_CLASS, 0/*don't care*/);

	ret = oc_ev_activate(ev_token, &my_ev_fd);
	if(ret){
		oc_ev_unregister(ev_token);
		exit(1);
	}

	for (i=0;i<10;i++) {
		struct timeval tv;
		 	
		FD_ZERO(&rset);
		FD_SET(my_ev_fd, &rset);
           	tv.tv_sec = 5;
              	tv.tv_usec = 0;


		if(select(my_ev_fd + 1, &rset, NULL,NULL,&tv) == -1){
			exit(1);
		}
		if(oc_ev_handle_event(ev_token)){
			exit(1);
		}
	}
	/* return 2. Logically I should exit with 1. But as it
	* stand there is another bug in ipc library, fails
	* the receives with EGAIN, even when the CCM daemon is
	* running. So we are returning 2. When that bug is
	* fixed, this ugly patch shall be removed.
	*/
	exit(2);
}
