/*
 *
 *   Copyright (c) International Business Machines  Corp., 2003
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Module: libmac.so
 *
 *   File: mac_plugin.h
 */

#ifndef   _MAC_HEADERS_
#define   _MAC_HEADERS_ 1


// these logging macros expect args ....
#define LOG_WARNING(args...) mac_eng_funcs->write_log_entry(WARNING,mac_plugin, ##args)
#define LOG_INFO(args...)    mac_eng_funcs->write_log_entry(DEFAULT,mac_plugin, ##args)
#define LOG_ERROR(args...)   mac_eng_funcs->write_log_entry(ERROR,mac_plugin, ##args)
#define LOG_DEBUG(args...)   mac_eng_funcs->write_log_entry(DEBUG,mac_plugin, ##args)
#define LOGENTRY()           mac_eng_funcs->write_log_entry(ENTRY_EXIT,mac_plugin,"%s: entry\n", __FUNCTION__ )
#define LOGEXIT()            mac_eng_funcs->write_log_entry(ENTRY_EXIT,mac_plugin,"%s: exit\n", __FUNCTION__ )
#define LOGEXITRC()          mac_eng_funcs->write_log_entry(ENTRY_EXIT,mac_plugin,"%s: exit, RC= %d\n", __FUNCTION__ , rc)

// used by option APIs to set string fields
#define SET_STRING_FIELD(a,b)\
a = mac_eng_funcs->engine_alloc( strlen(b)+1 );\
if (a ) {\
    strcpy(a, b);\
}\
else {\
    LOG_DEBUG("RC= ENOMEM\n");\
    LOGEXIT();\
    return ENOMEM;\
}

#define REQUIRE( x ) \
if (!(x)) { \
mac_eng_funcs->write_log_entry(ENTRY_EXIT,mac_plugin,"%s: exit, RC= EINVAL\n", __FUNCTION__ );\
return EINVAL;\
}

#define POPUP_MSG(answer,choices,args...) mac_eng_funcs->user_message(mac_plugin,answer,choices, ##args)

#define KILL_SECTORS(ld,lba,count) ((struct plugin_functions_s *)ld->plugin->functions.plugin)->add_sectors_to_kill_list( ld,lba,count);

#define DISK_READ(ld, lsn, count, buffer) ld->plugin->functions.plugin->read(ld, lsn, count, buffer)
#define DISK_WRITE(ld, lsn, count, buffer) ld->plugin->functions.plugin->write(ld, lsn, count, buffer)

#define FOR_EACH(item, list)    for ( rc = GoToStartOfList(list); \
                                      !rc && (item = get_object_from_list(list)); \
                                      rc = NextItem(list) )
      			 
/* Pretty typedefs */
typedef storage_object_t     LOGICALDISK ;
typedef storage_object_t     DISKSEG;


//
// MAC Disk Private Data Area
//
#define  MAC_DISK_PDATA_SIGNATURE          0x4D41432D  /* MAC- */

#define  DISK_HAS_CHANGES_PENDING          0x00000001
#define  DISK_HAS_MOVE_PENDING             0x00000002

#include "mac_disklabel.h"

typedef struct disk_private_data_s {

	u_int32_t          signature;		/* used to validate disk private data area           */
        
	u_int32_t          flags;		/* see DISK_HAS_xxxx flags above                     */

	u_int32_t          vsectors_per_block;	/* disk hardsector size >> EVMS_VSECTOR_SIZE         */
	
        u_int32_t          pcount;              /* number of partition map entries                   */

        copy_job_t        *copy_job;		/* ptr to valid copy job if a segment move is in     */
						/* progress ... DISK_HAS_MOVE_PENDING should be      */
						/* set if this is the case.                          */
        
        mac_partition_t    pmap;                /* copy of the pmap partition record                 */

} disk_private_data_t; 


//
// MAC Segment Private Data Area
//
#define  MAC_SEGMENT_PDATA_SIGNATURE       0x4D5A4547   /* MSEG */
#define  SEG_CFLAG_TOP_SEGMENT             0x00000001

typedef struct seg_private_data_s {

	u_int32_t          signature;		/* used to validate our private data area            */

	u_int32_t          cflags;		/* common flag definitions                           */

	//----------------- end of segment manager common area  -----------------------

	storage_object_t  *logical_disk;	/* storage object we found the mac partitions on     */

	mac_partition_t    p_record;		/* mac partition record we read off the disk label   */                                         

	u_int32_t          minor;		/* minor number assigned to this segment             */
	
        u_int32_t          pmap_index;          /* location of record in the PMAP                    */

        DISKSEG	          *move_target;         /* used when moving segments on the disk             */
    
} seg_private_data_t;


extern struct engine_functions_s       *mac_eng_funcs;
extern plugin_record_t                 *mac_plugin;

#include <common.h>
#include "disks.h"
#include "helpers.h"
#include "commit.h"
#include "options.h"
#include "discovery.h"
#include "segments.h"
#include "dm.h"

#endif

