/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Module: libos2regmgr.so
 *
 *   File: orm_dm.c
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <plugin.h>

#include "os2regmgr.h"
#include "orm_dm.h"


/*
 *  Function: orm_activate_compatibility
 *
 *  Called in order to create a mapping of a disk partition
 *  in the kernel.
 */
static int orm_activate_compatibility( storage_object_t *region )
{
        int rc=EINVAL;
        storage_object_t *seg; // os2 partition ... segment object
        dm_target_t target;
        dm_device_t linear;

        LOGENTRY();

	seg = orm_get_first_object_in_list(region->child_objects);
	
        if (seg) {
                target.start  = 0;
                target.length = region->size;
                target.type   = DM_TARGET_LINEAR;
                target.data.linear = &linear;
                target.params = NULL;
                target.next   = NULL;
                linear.major  = seg->dev_major;
                linear.minor  = seg->dev_minor;
                linear.start  = 0;
                rc = ORM_EngFncs->dm_activate(region, &target);
        }

        if (!rc) {
                region->flags |=  SOFLAG_ACTIVE;
                region->flags &= ~SOFLAG_NEEDS_ACTIVATE;
        }


        LOGEXITRC();
        return rc;
}


/*
 *  Function: orm_activate
 *
 *  Called in order to create a mapping of an os2 object
 *  in the kernel.
 */
int orm_activate( storage_object_t *region )
{
        int rc=EINVAL, i;
        orm_private_data_t *pdata=NULL;
        os2_drivelink_t    *dlink=NULL;
        dm_target_t *target, *target_list = NULL;
        dm_device_t *dev;

        LOGENTRY();

        pdata = (orm_private_data_t *)region->private_data;

        if ((pdata) && (pdata->signature == ORM_PDATA_SIGNATURE)) {

		if (pdata->flags & COMPATIBILITY_VOLUME) {
			rc = orm_activate_compatibility(region);
		}
		else {
			if (pdata->drive_link_count) {

				dlink = pdata->drive_link;

				for (i=0,rc=0; (!rc) && (i<pdata->drive_link_count); i++) {
					target = ORM_EngFncs->dm_allocate_target( DM_TARGET_LINEAR,
										dlink->start_sector,
										dlink->sector_count,
										0);
					if (target) {
						dev = target->data.linear;
						dev->major = dlink->object->dev_major;
						dev->minor = dlink->object->dev_minor;
						dev->start = 0;
						ORM_EngFncs->dm_add_target(target, &target_list);
					} else {
						rc = ENOMEM;
					}

					dlink = dlink->next;
				}

				if (!rc) {

					rc = ORM_EngFncs->dm_activate(region, target_list);

					if (!rc) {
						region->flags |=  SOFLAG_ACTIVE;
						region->flags &= ~SOFLAG_NEEDS_ACTIVATE;
					}

				}

				ORM_EngFncs->dm_deallocate_targets(target_list);
			}
		}
        }

        LOGEXITRC();
        return rc;
}


/*
 *  Function: orm_deactivate
 *
 *  Called in order to delete an existing mapping of an os2
 *  storage object in the kernel.
 */
int orm_deactivate( storage_object_t *region )
{
        int rc;

        LOGENTRY();

        rc = ORM_EngFncs->dm_deactivate(region);

        if (!rc) {
                region->flags &= ~SOFLAG_ACTIVE;
        }

        LOGEXIT();
        return rc;
}

